"use strict";
/**
 * @license
 * Copyright 2018 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tfc = require("@tensorflow/tfjs-core");
var utils_1 = require("./utils");
exports.executeOp = function (node, tensorMap, context) {
    switch (node.op) {
        case 'Abs':
        case 'ComplexAbs':
            return [tfc.abs(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Acos':
            return [tfc.acos(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Acosh':
            return [tfc.acosh(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Asin':
            return [tfc.asin(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Asinh':
            return [tfc.asinh(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Atan':
            return [tfc.atan(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Atan2':
            return [tfc.atan2(utils_1.getParamValue('x', node, tensorMap, context), utils_1.getParamValue('y', node, tensorMap, context))];
        case 'Atanh':
            return [tfc.atanh(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Ceil':
            return [tfc.ceil(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Complex':
            return [tfc.complex(utils_1.getParamValue('real', node, tensorMap, context), utils_1.getParamValue('imag', node, tensorMap, context))];
        case 'Cos':
            return [tfc.cos(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Cosh':
            return [tfc.cosh(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Elu':
            return [tfc.elu(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Erf':
            return [tfc.erf(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Exp':
            return [tfc.exp(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Expm1': {
            return [tfc.expm1(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Floor':
            return [tfc.floor(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Log':
            return [tfc.log(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Log1p': {
            return [tfc.log1p(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Imag':
            return [tfc.imag(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Neg':
            return [tfc.neg(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Reciprocal': {
            return [tfc.reciprocal(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Real':
            return [tfc.real(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Relu':
            return [tfc.relu(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Round': {
            return [tfc.round(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Selu':
            return [tfc.selu(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Sigmoid':
            return [tfc.sigmoid(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Sin':
            return [tfc.sin(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Sign': {
            return [tfc.sign(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Sinh': {
            return [tfc.sinh(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Softplus': {
            return [tfc.softplus(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Sqrt': {
            return [tfc.sqrt(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Square': {
            return [tfc.square(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Tanh': {
            return [tfc.tanh(utils_1.getParamValue('x', node, tensorMap, context))];
        }
        case 'Tan':
            return [tfc.tan(utils_1.getParamValue('x', node, tensorMap, context))];
        case 'Relu6':
        case 'ClipByValue':
            return [tfc.clipByValue(utils_1.getParamValue('x', node, tensorMap, context), utils_1.getParamValue('clipValueMin', node, tensorMap, context), utils_1.getParamValue('clipValueMax', node, tensorMap, context))];
        case 'Rsqrt':
            return [tfc.rsqrt(utils_1.getTensor(node.inputNames[0], tensorMap, context))];
        case 'Prod':
            return [tfc.prod(utils_1.getParamValue('x', node, tensorMap, context), utils_1.getParamValue('axes', node, tensorMap, context))];
        case 'LeakyRelu':
            return [tfc.leakyRelu(utils_1.getParamValue('x', node, tensorMap, context), utils_1.getParamValue('alpha', node, tensorMap, context))];
        case 'Prelu':
            return [tfc.prelu(utils_1.getParamValue('x', node, tensorMap, context), utils_1.getParamValue('alpha', node, tensorMap, context))];
        default:
            throw TypeError("Node type " + node.op + " is not implemented");
    }
};
exports.CATEGORY = 'basic_math';
//# sourceMappingURL=basic_math_executor.js.map